package XML::Comma::Pkg::DecoratorTest::SimpleTest;
use strict;
use warnings;

use base 'XML::Comma::Util::DefModule';
__PACKAGE__->load_def;

1;

__DATA__

<DocumentDefinition>
	<name>SimpleTest</name>
	<class><module>XML::Comma::Pkg::DecoratorTest::SimpleTest</module></class>
</DocumentDefinition>

